# -*- encoding: utf-8 -*-

import os
import flask
import uuid
from sqlalchemy.exc import IntegrityError
from flask_fas_openid import FAS, fas_login_required
from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker
import json
import logging
import subprocess

from . import models, forms, lib


os.environ.setdefault('INTEGRATOR_SETTINGS', '/etc/poormanci.conf')

app = flask.Flask(__name__)
app.config.from_object('poormanci.default_settings')
app.config.from_envvar('INTEGRATOR_SETTINGS', silent=True)
app.logger.setLevel(logging.INFO)


db_session = None


def connect_db():
    global db_session
    engine = create_engine(app.config['DATABASE'], convert_unicode=True)
    db_session = scoped_session(sessionmaker(autocommit=False,
                                             autoflush=False,
                                             bind=engine))
    models.Base.query = db_session.query_property()
    return db_session


fas = FAS(app)


@app.route('/')
def index():
    if flask.g.fas_user:
        projects = models.Project.query.filter(
            models.Project.owner == flask.g.fas_user.username).order_by(
                models.Project.name).all()
    else:
        projects = []
    return flask.render_template('index.html', projects=projects)


@app.route('/dashboard')
def dashboard():
    projects = models.Project.query.order_by(models.Project.name).all()
    mapping = []
    for project in projects:
        mapping.append((project, project.runs[-6:]))
    return flask.render_template('dashboard.html', projects=mapping)


@app.route('/login', methods=['GET', 'POST'])
def auth_login():
    index_url = flask.url_for('index')
    if flask.g.fas_user:
        return flask.redirect(index_url)
    return fas.login(groups=[])


@app.route('/logout')
def logout():
    if flask.g.fas_user:
        fas.logout()
    return flask.redirect(flask.url_for('index'))


@app.route('/help')
def help():
    return flask.render_template('help.html')


@app.route('/new', methods=['GET', 'POST'])
@fas_login_required
def new_project():
    form = forms.ProjectForm(flask.request.form)
    if flask.request.method == 'POST' and form.validate():
        token = uuid.uuid4().hex
        p = models.Project(
            name=form.name.data,
            display_name=form.display_name.data,
            owner=flask.g.fas_user.username,
            pagure_name=form.pagure_name.data,
            pagure_url=cleanup_url(form.pagure_url.data),
            pagure_token=form.pagure_token.data,
            jenkins_name=form.jenkins_name.data,
            jenkins_url=cleanup_url(form.jenkins_url.data),
            jenkins_token=form.jenkins_token.data,
            hook_token=token
        )
        db_session.add(p)
        try:
            db_session.commit()
            flask.flash('Project created', 'success')
            return flask.redirect(flask.url_for('index'))
        except IntegrityError:
            flask.flash('Error saving project. This name is already used.', 'danger')
    return flask.render_template('project_form.html',
                                 is_edit=False,
                                 form=form,
                                 title='Set up new project',
                                 submit='Create')


@app.route('/edit/<name>', methods=['GET', 'POST'])
@fas_login_required
def edit_project(name):
    p = models.Project.query.get(name)
    if not p:
        return flask.abort(404)
    if flask.g.fas_user.username != p.owner:
        return flask.abort(403)
    form = forms.ProjectForm(flask.request.form, obj=p)
    title = 'Update {name}'.format(name=name)
    if flask.request.method == 'POST' and form.validate():
        p.display_name = form.display_name.data
        p.pagure_name = form.pagure_name.data
        p.pagure_url = cleanup_url(form.pagure_url.data)
        p.pagure_token = form.pagure_token.data
        p.jenkins_name = form.jenkins_name.data
        p.jenkins_url = cleanup_url(form.jenkins_url.data)
        p.jenkins_token = form.jenkins_token.data

        db_session.add(p)
        db_session.commit()
        flask.flash('Project updated', 'success')
        return flask.redirect(flask.url_for('index'))
    return flask.render_template('project_form.html',
                                 is_edit=True,
                                 form=form,
                                 title=title,
                                 submit='Save')


@app.route('/delete/<name>', methods=['GET', 'POST'])
@fas_login_required
def delete_project(name):
    p = models.Project.query.get(name)
    if not p:
        return flask.abort(404)
    if flask.g.fas_user.username != p.owner:
        return flask.abort(403)
    if flask.request.method == 'POST':
        db_session.delete(p)
        db_session.commit()
        flask.flash('Project deleted', 'success')
        return flask.redirect(flask.url_for('index'))
    return flask.render_template('delete.html', p=p)


@app.route('/hooks/<token>/build-finished', methods=['POST'])
def hook_finished(token):
    try:
        data = json.loads(flask.request.get_data())
        cfg = models.get_configs(data['name'], models.Service.JENKINS)[0]
        build_id = data['build']['number']
        if token != cfg.hook_token:
            raise ValueError('Token mismatch')
    except (TypeError, ValueError, KeyError, models.ConfigNotFound) as exc:
        app.logger.error('Error processing jenkins notification', exc_info=exc)
        return ('Bad request...\n', 400, {'Content-Type': 'text/plain'})
    app.logger.info('Received jenkins notification')
    lib.process_build(db_session, app.logger, cfg, build_id)
    return ('', 204)


@app.route('/hooks/<token>/trigger-build', methods=['POST'])
def hook_trigger(token):
    pass


@app.errorhandler(404)
def not_found(e):
    return flask.render_template('not_found.html'), 404


@app.errorhandler(403)
def forbidden(e):
    return flask.render_template('forbidden.html'), 403


def cleanup_url(url):
    """Make sure there is trailing slash."""
    return url.rstrip('/') + '/'


@app.before_request
def before_request():
    if db_session is None:
        connect_db()

    if getattr(flask.g, 'version', None) is None:
        flask.g.version = get_version()


@app.teardown_appcontext
def shutdown_session(exception=None):
    db_session.remove()


def get_version():
    location = os.path.join(os.path.dirname(os.path.realpath(__file__)), '..')
    if os.path.isdir(os.path.join(location, '.git')):
        proc = subprocess.Popen(['git', '-C', location, 'describe', '--tags'],
                                stdout=subprocess.PIPE)
        output, _ = proc.communicate()
        return output.strip()
    else:
        import pkg_resources
        try:
            return pkg_resources.get_distribution('poor-man-ci').version
        except pkg_resources.DistributionNotFound:
            return 'unknown'
