# -*- encoding: utf-8 -*-

import sqlalchemy as sa
import sqlalchemy.orm as orm
from sqlalchemy.ext.declarative import declarative_base
from datetime import datetime


Base = declarative_base()


class Project(Base):
    __tablename__ = 'projects'

    name = sa.Column(sa.String(64), primary_key=True, unique=True)
    display_name = sa.Column(sa.String(64), nullable=False, default='Jenkins')
    owner = sa.Column(sa.String(64))

    pagure_name = sa.Column(sa.String(255))
    pagure_url = sa.Column(sa.String(255))
    pagure_token = sa.Column(sa.String(64))

    jenkins_name = sa.Column(sa.String(255))
    jenkins_url = sa.Column(sa.String(255))
    jenkins_token = sa.Column(sa.String(64))

    hook_token = sa.Column(sa.String(64))

    runs = orm.relationship('TestRun', order_by='TestRun.id')

    def __init__(self, name, display_name, owner,
                 pagure_name, pagure_url, pagure_token,
                 jenkins_name, jenkins_url, jenkins_token,
                 hook_token):
        self.name = name
        self.display_name = display_name
        self.owner = owner
        self.pagure_name = pagure_name
        self.pagure_url = pagure_url
        self.pagure_token = pagure_token

        self.jenkins_name = jenkins_name
        self.jenkins_url = jenkins_url
        self.jenkins_token = jenkins_token

        self.hook_token = hook_token

    def __repr__(self):
        return '<Project {.name}>'.format(self)


class TestRun(Base):
    __tablename__ = 'runs'

    id = sa.Column(sa.Integer, primary_key=True)
    project = sa.Column(sa.String(64), sa.ForeignKey('projects.name'))
    number = sa.Column(sa.Integer)
    timestamp = sa.Column(sa.DateTime())
    success = sa.Column(sa.Boolean, nullable=True)
    job = sa.Column(sa.Integer, nullable=True)

    def __init__(self, project, number):
        self.project = project
        self.number = number
        self.timestamp = datetime.now()
        self.success = None
        self.job = None

    @property
    def status(self):
        if self.success is None:
            return 'pending'
        return 'success' if self.success else 'failure'

    def __repr__(self):
        return '<TestRun {.id}>'.format(self)


def create_run(session, project, pr_id):
    run = TestRun(project, pr_id)
    session.add(run)
    session.commit()
    return run.id


def mark_run(session, ident, status, job):
    """Mark a run as finished in database and return it. If result is already
    saved, returns None.
    """
    run = TestRun.query.get(ident)
    if status == run.success:
        return run, False
    run.success = status
    run.job = job
    session.add(run)
    session.commit()
    return run, True


def init_db(db):
    from sqlalchemy import create_engine
    engine = create_engine(db, convert_unicode=True)
    Base.metadata.create_all(bind=engine)


class ConfigNotFound(Exception):
    pass


class Service(object):
    PAGURE = Project.pagure_name
    JENKINS = Project.jenkins_name


def get_configs(project_name, service):
    """Returns all configurations with given name on a service.

    :raises ConfigNotFound: when no configuration matches
    """
    cfg = Project.query.filter(service == project_name).all()
    if len(cfg) == 0:
        raise ConfigNotFound(project_name)
    return cfg
